----------------------------------------------------------------------------------
-- Company: Digilent Ro
-- Engineer: Cristian Fatu
-- 
-- Create Date:    2014 March 14 
-- Design Name: 
-- Module Name:    TFTCtl - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: TFTCtl generates the TFT backlight PWM signal, the LCD clock 
--
-- Dependencies:
--
-- Revision: 
-- Revision 0.01 - File Created, using old TftCtl file created by Elod Gyorgy
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity TFTSigCtl is

	generic ( 
            constant H_MAX : NATURAL := 889;
			constant V_MAX : NATURAL := 513
				);
   Port (
		CLK_I : in STD_LOGIC;		
		CLK_O : out  STD_LOGIC;
		DISP_O : out  STD_LOGIC;
		BKLT_O : out  STD_LOGIC		
	);
end TFTSigCtl;

architecture Behavioral of TFTSigCtl is

signal dataBus: std_logic_vector(15 downto 0);

constant CLOCKFREQ : natural := 50; --MHZ
constant TLEDWARMUP : natural := 100; --ms
constant TLEDCOOLDOWN : natural := 80; --ms
constant TVDDRAMP : natural := 1; --ms
constant TLEDWARMUP_CYCLES : natural := natural(CLOCKFREQ*TLEDWARMUP);
constant TLEDCOOLDOWN_CYCLES : natural := natural(CLOCKFREQ*TLEDCOOLDOWN);
constant TVDDRAMP_CYCLES : natural := natural(CLOCKFREQ*TVDDRAMP);

signal waitCnt : natural range 0 to TLEDWARMUP_CYCLES := 0;



type state_type is (stOff, stVddRamp, stLEDWarmup, stLEDCooldown, stOn); 
   signal state , nstate : state_type := stOff;

signal int_Bklt, clkRst, dispActive : std_logic;

	
	COMPONENT PWM
   Generic (
		C_CLK_I_FREQUENCY : natural := CLOCKFREQ; -- in MHZ
		C_PWM_FREQUENCY : natural := 20000; -- in Hz
		C_PWM_RESOLUTION : natural := 3 
	);
	PORT(
		CLK_I : IN std_logic;
		RST_I : IN std_logic;
		DUTY_FACTOR_I : in STD_LOGIC_VECTOR (C_PWM_RESOLUTION-1 downto 0);        
		PWM_O : OUT std_logic
		);
	END COMPONENT;
	
begin

------------------------------------------------------------
-- PWM controlling the backlight intensity
----------------------------------------------------------------------------------
	Inst_PWM: PWM 
	generic map (
		C_CLK_I_FREQUENCY => CLOCKFREQ, -- in MHZ
		C_PWM_FREQUENCY => 25000, -- in Hz
		C_PWM_RESOLUTION => 3
	)
	PORT MAP(
		CLK_I => CLK_I,
		RST_I => '0',
		PWM_O => int_Bklt,
		DUTY_FACTOR_I => "111"
	);
	
	
----------------------------------------------------------------------------------
-- Clock Forwarding done right
-- This will forward the internal pixel clock to the TFT clock pin
----------------------------------------------------------------------------------
   ODDR_inst : ODDR
   generic map(
      DDR_CLK_EDGE => "OPPOSITE_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE" 
      INIT => '0',   -- Initial value for Q port ('1' or '0')
      SRTYPE => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
   port map (
      Q => CLK_O,   -- 1-bit DDR output
      C => CLK_I,    -- 1-bit clock input
      CE => '1',  -- 1-bit clock enable input
      D1 => '1',  -- 1-bit data input (positive edge)
      D2 => '0',  -- 1-bit data input (negative edge)
      R => clkRst,    -- 1-bit reset input
      S => '0'     -- 1-bit set input
   );
	
 clkRst <= not dispActive;

   SYNC_PROC: process (CLK_I)
   begin
      if (CLK_I'event and CLK_I = '1') then
            state <= nstate;       
      end if;
   end process;	

	NEXT_STATE_DECODE: process (state, waitCnt)
   begin
      nstate <= state;
      case (state) is
         when stOff =>
               nstate <= stVddRamp;
			when stVddRamp =>
				if (waitCnt = TVDDRAMP_CYCLES) then
					nstate <= stLEDWarmup;
				end if;
         when stLEDWarmup =>
            if (waitCnt = TLEDWARMUP_CYCLES) then
               nstate <= stOn;
            end if;
         when stOn =>
			when stLEDCooldown =>
            if (waitCnt = TLEDCOOLDOWN_CYCLES) then
               nstate <= stOff;
            end if;
      end case;      
   end process;
	
	process(CLK_I)
   begin
	if Rising_Edge(CLK_I) then
		if state /= nstate then
			waitCnt <= 0;
		else
			waitCnt <= waitCnt + 1;
		end if;
	end if;
end process;


----------------------------------------------------------------------------------
-- LCD Power Sequence
----------------------------------------------------------------------------------	
dispActive <= 	'0' when state = stOff or state = stVddRamp else
					'1';
--Backlight adjust/enable
BKLT_O <= 	int_Bklt when state = stOn else
				'0';
--Display On/Off signal
DISP_O <= dispActive;	



end Behavioral;

